
#ifndef READ_PHYTREE_HPP
#define READ_PHYTREE_HPP

#include <iostream>
#include <string>

#include "utils/phylo_typedefs.hpp"

namespace io {
  static std::string delimeters("\t\n ,");
  static std::string open_brackets("([{");
  static std::string close_brackets("}])");
  static std::string newlines("\r\n");
  static std::string non_name_chars(delimeters + open_brackets + close_brackets);

  // read a tree in bracket notation and return success
  bool read_rooted_tree_from_brackets_internal(PhyloNet& N, const PhyloNet::InternVertex& parent, std::istream& in){
    // eat leading spaces
    in >> std::ws;
    // if we read eof prematurely (or the string is empty), return failure
    if(in.eof()) return false;
    // prepare vertex
    PhyloNet::InternVertex root = N.add_vertex(parent);
    // either read an open bracket and recurse or read the name of the leaf
    int c;
    if(open_brackets.find(in.peek()) != std::string::npos){
      do{
        // consume open bracket or delimeter and skip whitespces
        in.get();
        in >> std::ws;
        // then recurse
        if(!read_rooted_tree_from_brackets_internal(N, root, in)) return false;
        // until we find no more delimeters
      } while(delimeters.find(in.peek()) != std::string::npos);
      // skip whitespaces to the closing bracket
      in >> std::ws;
      // consume the closing bracket
      c = in.get();
      if(close_brackets.find(c) == std::string::npos) return false;
    } else {
      // if we do not have children, we are a leaf
      std::string name;
      do{
        c = in.peek();
        if(non_name_chars.find(c) != std::string::npos) break;
        in.get();
        name += (char)c;
      } while(true);
      N.make_leaf(root, name);
    }// if
    return true;
  }// function
  // version without the index reference
  bool read_rooted_tree_from_brackets(PhyloNet& N, std::istream& in){
    return read_rooted_tree_from_brackets_internal(N, N.set_root(), in);
  }
}


#endif

