
#pragma once


#include <iostream>
#include "utils/instance.hpp"
#include "utils/scaffolding_typedefs.hpp"
#include "utils/command_line.hpp"
#include "io/dot.hpp"

namespace io {

    Instance* read_instance(std::ifstream& infile,
                            const GeneralOptions& options,
                            SequenceMap* sequence_orientations = NULL)
    {
      Instance* result;
      if(infile.is_open()){
        DEBUG1(std::cout << "reading input file" << std::endl);
        // try dot first
        result = io::read_dot(infile, options, sequence_orientations);
        if(result == NULL)
          DEBUG1(std::cout << "Nothing read. Verify that the input file is in DOT format."<<std::endl);
        infile.close();
        return result;
      } else return NULL;
    }

    Instance* read_instance(std::ifstream& infile, SequenceMap* sequences = NULL)
    {
      return read_instance(infile, GeneralOptions(), sequences);
    }
    Instance* read_instance(const GeneralOptions& options, SequenceMap* sequence_orientations = NULL)
    {
      std::ifstream in(options.in_graph_filename);
      return read_instance(in, options, sequence_orientations);
    }
    
    void write_scaffold(std::ostream& outfile, const ScaffoldGraph& sg)
    {
      write_scaffold_as_dot(outfile, sg.get_graph());
    }

}// namespace

