/*
 * preprocess a given graph for linearization
*/

/* ----------------------------------------------------------------- */

#pragma once

#include "utils/graph_typedefs.hpp"
#include "utils/scaffold_graph.hpp"


namespace scaffold{
  namespace preprocess {


    void contract_uniform_paths(ScaffoldGraph &g) {
      std::vector<ScafEdge> non_matching_edges;

      for(auto inc_iter = g.get_non_matching_edges(); inc_iter; ++inc_iter)
        non_matching_edges.push_back(*inc_iter);

      for(const ScafEdge& ux: non_matching_edges) {
        const ScafVertex& u = g.source(ux);
        const ScafVertex& x = g.target(ux);
        const ScafEdge uv = g.incident_matching_edge(u);
        const ScafEdge xy = g.incident_matching_edge(x);
        const ScafVertex& v = g.target(uv);
        const ScafVertex& y = g.target(xy);

        if(!g.adjacent(v,y) && (g[ux].multiplicity == g[uv].multiplicity && g[ux].multiplicity == g[xy].multiplicity)){
          const LengthType xy_len = g[xy].length;
          const LengthType ux_len  = g[ux].length;
          const LengthType uv_len = g[uv].length;
          const MultiType xy_mul = g[xy].multiplicity;
          contract_non_contig(g, v, u, x, y, xy_len + ux_len + uv_len, xy_mul, false); // contract ux
        }
      }
    }

    void split_non_ambiguous_paths(ScaffoldGraph &g) {
      std::vector<ScafEdge> non_ambiguous_path;
      for(auto inc_iter = g.get_matching_edges(); inc_iter; ++inc_iter) {
        const ScafEdge uv = *inc_iter;
        const ScafVertex u = g.source(uv);
        const ScafVertex v = g.target(uv);
        if(g.degree(u) == 2 && g.degree(v) == 2){
          const ScafEdge& xu = *(g.get_incident_non_matching(u));
          const ScafEdge& vy = *(g.get_incident_non_matching(v));
          if(g[xu].multiplicity == g[uv].multiplicity && 
              g[vy].multiplicity == g[uv].multiplicity)
            non_ambiguous_path.push_back(uv);
        }
      }

      for(const ScafEdge& e: non_ambiguous_path) {
        const ScafVertex& u = g.source(e);
        const ScafVertex& v = g.target(e);
        const ScafEdgeProperty ep = g[e];
        g.delete_edge(e);
        const ScafVertex& u2 = g.add_vertex(g[u].name+"'");
        const ScafVertex& v2 = g.add_vertex(g[v].name+"'");
        g.add_matching_edge(u,u2,ep);
        g.add_matching_edge(v,v2,ep);
      }
    }


    void remove_deg1(ScaffoldGraph& sg)
    {
      auto u_it = sg.get_vertices();
      while(u_it){
        const ScafVertex& u = *u_it;
        if(sg.degree(u) == 1){
          const ScafVertex& v = sg.matched_with(u);
          if(sg.degree(v) == 1){
            sg.delete_vertex(v, false, false);
            sg.delete_vertex(*(u_it++), false, false);
          } else if(sg.degree(v) == 2){
            const ScafEdge& vw = *(sg.get_incident_non_matching(v));
            const ScafVertex& w = sg.target(vw);
            const ScafEdge& wx = sg.incident_matching_edge(w);
            const ScafVertex& x = sg.target(wx);
            if((sg[vw].multiplicity == sg[wx].multiplicity) || (sg.degree(x) == 1)){
              sg.delete_vertex(v, false, false);
              sg.delete_vertex(*(u_it++), false, false);
            } else ++u_it;
          } else ++u_it;
        } else ++u_it;
      }
      sg.update_vertex_indices();
    }

  } // namespace
} // namespace

