
#pragma once

#include <string>
#include <vector>

#ifdef STATISTICS
#include "utils/stats.hpp"
#endif

#include "utils/ilp_common.hpp"
#include "utils/ilp_options.hpp"
#include "utils/profiling.hpp"

#define VAR_COLLECTION var_collection_3layered

namespace scaffold{ namespace solv{
  
  using AuxGraph = AuxiliaryGraph<>;

  struct VAR_COLLECTION : public var_collection {
    using var_collection::var_collection;
    // variables for the second graph, containing the cycles
    VertexVarMap from_source_C;
    EdgeVarMap times_used_C;
    EdgeVarMap times_used_C_2to3;
  };


  // block isolated cycles as lazy constraint callback
  class LazyCyclesBlockCallback: public IloCplex::LazyConstraintCallbackI{
    const VAR_COLLECTION& vars;
    CallbackStatistics& stats;
  public:
    typedef TranslateBiMap<RawScaffoldGraph, AuxGraph> RawToAuxBiMap;
    typedef typename RawToAuxBiMap::value_type TranslatePair;

    IloCplex::CallbackI* duplicateCallback() const{
      return (new (getEnv()) LazyCyclesBlockCallback(getEnv(), vars, stats));
    }
    
    LazyCyclesBlockCallback(IloEnv env, const VAR_COLLECTION& _vars, CallbackStatistics& _stats):
      IloCplex::LazyConstraintCallbackI(env),
      vars(_vars),
      stats(_stats)
    {}
    
    // adds an arc to the auxiliary graph, given an IloNumVar and a VertexPair
    void add_arc_from_variable(
        const IloNumVar& var,
        const ScafVertexPair& uv,
        const RawToAuxBiMap& translate,
        AuxGraph& aux) const
    {
      const Instance& I = vars.I;
      if(is_true(getValue(var))){
        const ScafVertex u = uv.first;
        const ScafVertex v = uv.second;
        const Vertex<AuxGraph> new_u = translate.left.at(u);
        const Vertex<AuxGraph> new_v = translate.left.at(v);
        boost::add_edge(new_u, new_v, aux);
        DEBUG5(std::cout << "adding  "<<I.get_edge_name(uv)<<std::endl); 
      } else DEBUG5(std::cout << "not adding "<<I.get_edge_name(uv)<< " since "<<getValue(var)<<" translates to "<<is_true(getValue(var))<<std::endl);
    }

    void output_current_aux_graph(const AuxGraph& aux, const RawToAuxBiMap& translate) const
    {
    //void LazyCyclesBlockCallback::output_current_aux_graph(const AuxGraph& aux, const RawToAuxBiMap& translate) const{
      const Instance& I = vars.I;
      for(VertexIterRange<AuxGraph> i = boost::vertices(aux); i.first != i.second; ++i.first)
        if(boost::out_degree(*(i.first), aux) != boost::in_degree(*(i.first), aux)){
          const ScafVertex u = translate.right.at(*(i.first));

          std::cout << "found vertex "<< I[u].name<<" with outdeg = "<< out_degree(*(i.first), aux)<<" & indeg = "<< in_degree(*(i.first), aux) << std::endl;
          std::cout << "we have: ";
          if( is_true(getValue(vars.from_source.at(u)))) std::cout<<"s ";
          if( is_true(getValue(vars.from_source_C.at(u)))) std::cout<<"sc ";
          if( is_true(getValue(vars.to_path_sink.at(u)))) std::cout<<"tp ";
          if( is_true(getValue(vars.to_cycle_sink.at(u)))) std::cout<<"tc ";
          for(VertexIterRange<AuxGraph> j = vertices(aux); j.first != j.second; ++j.first){
            const ScafVertex v = translate.right.at(*(j.first));
            ScafVertexPair uv(u, v);
            ScafVertexPair vu(v, u);
            
            if(contains(vars.times_used, uv) && is_true(getValue(vars.times_used.at(uv)))) std::cout << "y_"<<I.get_edge_name(uv)<<", ";
            if(contains(vars.times_used, vu) && is_true(getValue(vars.times_used.at(vu)))) std::cout << "y_"<<I.get_edge_name(vu)<<", ";
            if(contains(vars.times_used_C, uv) && is_true(getValue(vars.times_used_C.at(uv)))) std::cout << "yc_"<<I.get_edge_name(uv)<<", ";
            if(contains(vars.times_used_C, vu) && is_true(getValue(vars.times_used_C.at(vu)))) std::cout << "yc_"<<I.get_edge_name(vu)<<", ";
            if(contains(vars.times_used_C_2to3, uv) && is_true(getValue(vars.times_used_C_2to3.at(uv)))) std::cout << "yc3_"<<I.get_edge_name(uv)<<", ";
            if(contains(vars.times_used_C_2to3, vu) && is_true(getValue(vars.times_used_C_2to3.at(vu)))) std::cout << "yc3_"<<I.get_edge_name(vu)<<", ";
          }// for
          std::cout << std::endl;
          exit(1);
      }// if & for
    }

    void output_solution_edges()
    {
      const Instance& I = vars.I;
      std::cout << "solution edges: "<<std::endl;
      for(auto e_it = I.get_edges(); e_it; ++e_it){
        const ScafEdge& e = *e_it;
        const ScafVertex& u = I.source(e);
        const ScafVertex& v = I.target(e);
        // get the appropriate x_ij variable
        const ScafVertexPair uv(u, v);
        const ScafVertexPair vu(v, u);
        std::cout<<I.get_edge_name(uv)<<" by ";

        if(contains(vars.times_used, uv) && is_true(getValue(vars.times_used.at(uv)))) std::cout << "y_"<<I.get_edge_name(uv)<<", ";
        if(contains(vars.times_used, vu) && is_true(getValue(vars.times_used.at(vu)))) std::cout << "y_"<<I.get_edge_name(vu)<<", ";
        if(contains(vars.times_used_C, uv) && is_true(getValue(vars.times_used_C.at(uv)))) std::cout << "yc_"<<I.get_edge_name(uv)<<", ";
        if(contains(vars.times_used_C, vu) && is_true(getValue(vars.times_used_C.at(vu)))) std::cout << "yc_"<<I.get_edge_name(vu)<<", ";
        if(contains(vars.times_used_C_2to3, uv) && is_true(getValue(vars.times_used_C_2to3.at(uv)))) std::cout << "yc3_"<<I.get_edge_name(uv)<<", ";
        if(contains(vars.times_used_C_2to3, vu) && is_true(getValue(vars.times_used_C_2to3.at(vu)))) std::cout << "yc3_"<<I.get_edge_name(vu)<<", ";
        std::cout << std::endl;
      }
    }

    void build_aux_graph(AuxGraph& aux, RawToAuxBiMap& translate)
    {
      // create vertices
      const Instance& I = vars.I;
      for(auto i = I.get_vertices(); i; ++i){
        const ScafVertex orig_u = *i;
        const Vertex<AuxGraph> new_u = boost::add_vertex(aux);
        translate.insert(TranslatePair(orig_u, new_u));
        DEBUG5(std::cout << "adding  "<< I[orig_u].name<<std::endl);
      }
      // create arcs,
      DEBUG5(std::cout << "adding y arcs"<<std::endl);
      for( const auto& i : vars.times_used )
        add_arc_from_variable(i.second, i.first, translate, aux);
      DEBUG5(std::cout << "adding y^c arcs"<<std::endl);
      for( const auto& i : vars.times_used_C)
        add_arc_from_variable(i.second, i.first, translate, aux);
      for( const auto& i : vars.times_used_C_2to3)
        add_arc_from_variable(i.second, i.first, translate, aux);
    }

    void get_pending(std::stack<Vertex<AuxGraph>>& pending, const RawToAuxBiMap& translate)
    {
      const Instance& I = vars.I;
      DEBUG5(std::cout<<"coming from s: ");
      for( const auto& i : vars.from_source )
        if(is_true(getValue(i.second))) {
          DEBUG5(std::cout <<  I[i.first].name<<" ");
          pending.push(translate.left.at(i.first));
        }
      DEBUG5(std::cout<<std::endl);
      
      DEBUG5(std::cout<<"coming from sc: ");
      for( const auto& i : vars.from_source_C )
        if(is_true(getValue(i.second))) {
          DEBUG5(std::cout <<  I[i.first].name<<" ");
          pending.push(translate.left.at(i.first));
        }
      DEBUG5(std::cout<<std::endl);

      DEBUG5({std::cout<<"going to tp: ";
        for( const auto& i : vars.to_path_sink )
          if(is_true(getValue(i.second))) std::cout <<  I[i.first].name<<" ";
        std::cout<< std::endl<<"going to tc: ";
        for( const auto& i : vars.to_cycle_sink )
          if(is_true(getValue(i.second))) std::cout <<  I[i.first].name<<" ";
        std::cout<<std::endl;});
    }

    // remove all paths from s from the current aux graph
    void remove_paths_from_s(AuxGraph& aux, const RawToAuxBiMap& translate)
    {
      std::stack<Vertex<AuxGraph> > pending;
      get_pending(pending, translate);

      const Instance& I = vars.I;
      while(!pending.empty()){
        const Vertex<AuxGraph> u = pending.top(); pending.pop();
        // note that u has an incoming arc y_s -> u, so, by flow conservation, it has only 1 outgoing arc
        const unsigned char deg = boost::out_degree(u, aux);
        assert(deg <= 1);
        // if this outgoing arc is not to y_t, then add the next vertex to pending & remove the arc
        if(deg == 1) {      
          const Edge<AuxGraph> e = *(boost::out_edges(u, aux).first);
          const Vertex<AuxGraph> v = boost::target(e, aux);
          DEBUG5(std::cout <<  I[translate.right.at(v)].name << " is on a path by edge from "
                           << I[translate.right.at(boost::source(e, aux))].name<<std::endl);
          pending.push(v);
          boost::remove_edge(e, aux);
        }
      }

    }

    void main()
    {
    //void LazyCyclesBlockCallback::main(){

      DEBUG5(output_solution_edges());

      // resume timer
      stats.time_spent.resume();

      // prepare aux graph
      const Instance& I = vars.I;
      AuxGraph aux;

      // Step 1: construct solution graph
      RawToAuxBiMap translate;
      build_aux_graph(aux, translate);
      
      // Step 2: in order to detect isolated cycles, remove all paths starting in s
      remove_paths_from_s(aux, translate);

      // Step 3: find a/all cycle in the remaining part of the graph
      DEBUG5(std::cout<< "remaining arcs in auxilliary graph: "<<num_edges(aux)<<std::endl;);
      unsigned num_new_cuts = 0;
      if(num_edges(aux)){
        DEBUG5(output_current_aux_graph(aux, translate));

#ifdef LAZY_ADD_ALL_CYCLES
      while(num_edges(aux)){
#endif
        std::stack<Edge<AuxGraph> > e_pending;
        e_pending.push(*(edges(aux).first));
        IloExpr p_expr(getEnv()), rev_p_expr(getEnv()); // forbid the cycle and its reverse in D_p
        IloExpr c_expr(getEnv()), rev_c_expr(getEnv()); // forbid the cycle and its reverse in D_c
        unsigned count = 0;
        DEBUG3(std::cout << "forbidding cycle: ");
        while(!e_pending.empty()){
          const Edge<AuxGraph> e = e_pending.top(); e_pending.pop();
          const Vertex<AuxGraph> u = boost::source(e, aux);
          const Vertex<AuxGraph> v = boost::target(e, aux);
          const ScafVertex old_u = translate.right.at(u);
          const ScafVertex old_v = translate.right.at(v);
          const ScafVertexPair old_uv = ScafVertexPair(old_u, old_v);
          const ScafVertexPair old_vu = ScafVertexPair(old_v, old_u);

          DEBUG3(std::cout << I.get_edge_name(old_uv)<<" ");
          boost::remove_edge(e, aux);
          const unsigned char v_deg = boost::out_degree(v, aux);
          assert(v_deg <= 1);
          // if this outgoing arc is not to our initial arc, then add the next vertex to pending & remove the arc
          if(v_deg == 1) e_pending.push(*(out_edges(v, aux).first));
          
          // forbid the cycle and its reverse in D_p & D_c
          p_expr += vars.times_used.at(old_uv);
          c_expr += vars.times_used_C_2to3.at(old_uv);
                  
          rev_p_expr += vars.times_used.at(old_vu);
          rev_c_expr += vars.times_used_C_2to3.at(old_vu);

          ++count;
        }
        DEBUG5(std::cout << " by allowing only <"<<count<<" of its edges");
        DEBUG3(std::cout << std::endl);
        add(p_expr < count);
        add(c_expr < count);
        add(rev_p_expr < count);
        add(rev_c_expr < count);
        num_new_cuts += 4;
#ifdef LAZY_ADD_ALL_CYCLES
      }// while(num_edges(aux))
#endif
      }// if(num_edges(aux))
      // Step 4: collect statistical information
      DEBUG2(std::cout << "=======   callback called, added "<<num_new_cuts<<" new cuts ======"<<std::endl);
      stats.times_called++;
      stats.cuts_added += num_new_cuts;
      stats.time_spent.pause();
    } // main()


  }; // class

  // callback function
  IloCplex::Callback GetLazyCyclesBlock(IloEnv env, const VAR_COLLECTION& vars, CallbackStatistics& stats) {
    return (new (env) LazyCyclesBlockCallback(env, vars, stats));
  }




  // add a variable for an edge
  void create_edge_variable(
      IloEnv& env,
      EdgeVarMap& x,
      const IloNumVar::Type& Type,
      const size_t lower_bnd,
      const size_t upper_bnd,
      const Instance& I,
      const ScafVertex& u,
      const ScafVertex& v,
      const std::string& prefix)
  {
    const std::string full_name(prefix + I[u].name + "," + I[v].name);
    DEBUG5(std::cout << "creating "<<full_name<<std::endl);
    x.emplace(PWC, std::make_tuple(u,v), std::make_tuple(env, lower_bnd, upper_bnd, Type, full_name.c_str()));
  }


  // initialize the variables for the ILP formulation
  void populate_variables(IloModel& model,
                          IloEnv& env,
                          VAR_COLLECTION& vars,
                          const ILPOptions& opts
#ifdef STATISTICS
                          ,JumpStatistics& jump_stats
#endif
                          )
  {
    const Instance& I = vars.I;
    DEBUG5(std::cout << "creating x_{u,v} and y_uv"<<std::endl);
    for(auto e = I.get_edges(); e; ++e){
      const ScafVertex& u = I.source(*e);
      const ScafVertex& v = I.target(*e);
      //const std::string& uname = I[u].name;
      //const std::string& vname = I[v].name;
      
      // Step 1: add variables x_e = 1   <=>  e belogs to a solution path/cycle
      create_edge_variable(env, vars.times_used_undir, IloNumVar::Bool, 0, 1, I, u, v, "x_");
      // Step 2: add variables y_ij and yc_ij
      create_edge_variable(env, vars.times_used, IloNumVar::Bool, 0, 1, I, u, v, "y[1<->2]_");
      create_edge_variable(env, vars.times_used, IloNumVar::Bool, 0, 1, I, v, u, "y[1<->2]_");
      create_edge_variable(env, vars.times_used_C, IloNumVar::Bool, 0, 1, I, u, v, "yc[1<->2]_");
      create_edge_variable(env, vars.times_used_C, IloNumVar::Bool, 0, 1, I, v, u, "yc[1<->2]_");

      // Step 3: add variables y^c_ij for D_c
      if(!I[*e].is_matching_edge()){
        // non-matching edges go from layer 1 to layer 2 in D_c
        create_edge_variable(env, vars.times_used_C_2to3, IloNumVar::Bool, 0, 1, I, u, v, "y[2->3]_");
        create_edge_variable(env, vars.times_used_C_2to3, IloNumVar::Bool, 0, 1, I, v, u, "y[2->3]_");
      }
    }
    DEBUG5(std::cout << "creating s/tp/tc interaction variables"<<std::endl);
    // Step 3: add variables y_s,i and y_i,tp and y_i,tc
    for(auto v_it = I.get_vertices(); v_it; ++v_it){
      const ScafVertex& v = *v_it;
      const std::string& name = I[v].name;
      // on layer 0, come up from s & sc
      vars.from_source[v] = IloNumVar(env, 0, 1, IloNumVar::Bool, ("y_s," + name).c_str());
      vars.from_source_C[v] = IloNumVar(env, 0, 1, IloNumVar::Bool, ("yc_s," + name).c_str());
      // on layer 1, go up to tp
      vars.to_path_sink[v] = IloNumVar(env, 0, 1, IloNumVar::Bool, ("y_" + name + ",tp").c_str());
      // on layer 2, go up to tc
      vars.to_cycle_sink[v] = IloNumVar(env, 0, 1, IloNumVar::Bool, ("y_" + name + ",tc").c_str());
    }

    if(!opts.ignore_contig_jumps){
      // constraint: contig jumps
      DEBUG4(std::cout << "creating contig-jump variables & expressions"<<std::endl);
      STAT(VertexPairToInt<RawScaffoldGraph> network_sizes);
      // Step 3: construct a contig jump network for uv
      for(auto e_it = I.get_non_matching_edges(); e_it; ++e_it){
        const ScafEdge& uv_edge = *e_it;
        const int static_length = I[uv_edge].length;
        if(static_length != NO_LENGTH){
          const ScafVertex& u = I.source(uv_edge);
          const ScafVertex& v = I.target(uv_edge);
          const UncertainLength uv_length = I.get_uncertain_length(uv_edge);
          const unsigned uv_multi = 1; // note that we are in the multiplicity-1 case!
          //const unsigned uv_multi = I[uv_edge].multiplicity;
          for(const ScafVertexPair uv: {ScafVertexPair(u,v), ScafVertexPair(v,u)}){
            // NOTE: the ILPContigJumpNetwork is only constructed to be integrated in the model
            // after integration it does not serve any more purpose and is discarded
            ILPContigJumpNetwork jump_network(I, uv, opts.jump_threshold_probability, uv_length);
            if(!jump_network.empty()){
              const std::string name = "jump_" + std::to_string(I.get_edge_name(uv));
              const IloNumVar& main_jump_var = vars.contig_jumps.emplace(PWC,
                  std::make_tuple(uv), std::make_tuple(env, 0, uv_multi, IloNumVar::Int, name.c_str())).first->second;
              jump_network.integrate_into_model(env, model, uv_multi, main_jump_var, vars.times_used);
              jump_network.integrate_into_model(env, model, uv_multi, main_jump_var, vars.times_used_C);
            }
            STAT(if(jump_network.num_edges() > 10) std::cout << "adding size-"<<jump_network.num_edges()<<" network"<<std::endl;)
            STAT(network_sizes[uv] = jump_network.num_edges();)
          }
        }
      }
      STAT(jump_stats.jump_network_sizes.from_pairs(network_sizes);)
    }// if

  }

  // initialize the constraints for the ILP formulation
  void populate_constraints(IloEnv& env, IloRangeArray& c, const VAR_COLLECTION& vars){
    const Instance& I = vars.I;

    // constraint (1'): of all the representations of ij, exactly x_ij should be chosen!
    DEBUG5(std::cout << "adding (1): of all the representations of e, exactly x_e should be chosen"<<std::endl);
    for(auto x : vars.times_used_undir){
      const ScafVertexPair& uv = x.first;
      const ScafVertexPair vu(uv.second, uv.first);

      IloExpr expr(env);
      expr += vars.times_used.at(uv);
      expr += vars.times_used.at(vu);
      expr += vars.times_used_C.at(uv);
      expr += vars.times_used_C.at(vu);

      if(!I.is_matching_edge(uv)){
        expr += vars.times_used_C_2to3.at(uv);
        expr += vars.times_used_C_2to3.at(vu);
      }
      expr -= x.second;
      c.add(expr == 0);
    }

    // constraint (2): all matching-edges should be chosen!
    DEBUG5(std::cout << "adding (2): all matching-edges should be chosen"<<std::endl);
    for(auto x : vars.times_used_undir)
      if(I.is_matching_edge(x.first)) c.add(x.second == 1);
  
    // constraint (4),(6): there should be at most sigma_p paths & sigma_c cycles
    DEBUG5(std::cout << "adding (4)&(6): there should be at most sigma_p paths & sigma_c cycles"<<std::endl);
    {
      IloExpr p_expr(env), c_expr(env);
      for(auto y : vars.to_path_sink) p_expr += y.second;
      c.add(p_expr <= (double)I.num_paths);
      for(auto y : vars.to_cycle_sink) c_expr += y.second;
      c.add(c_expr <= (double)I.num_cycles);
    }

    // constraint (3): everything that comes into layer l should leave layer l
    DEBUG5(std::cout << "adding (3): flow conservation"<<std::endl);
    for(unsigned layer = 0; layer < 2; ++layer){
      for(auto u = I.get_vertices(); u; ++u){
        IloExpr p_expr(env);
        IloExpr c_expr(env);
        for(auto e_uv = I.get_incident(*u); e_uv; ++e_uv){
          const ScafVertex& v = I.target(*e_uv);
          const ScafVertexPair uv(*u, v);
          // remember: layer 0 -> 1 contains only matching edges
          if(I[*e_uv].is_matching_edge() == (layer == 0)){
            // flow goes to next layer along uv
            DEBUG5(std::cout << " + y_" << I.get_edge_name(uv));
            p_expr += vars.times_used.at(uv);
            c_expr += vars.times_used_C.at(uv);

            // on layer 1, flow goes to layer 2
            if(layer == 1) c_expr += vars.times_used_C_2to3.at(uv);
          } else {
            const ScafVertexPair vu(v, *u);
            // flow comes from the other layer alog vu
            DEBUG5(std::cout << " - y[1<->2]"<<I.get_edge_name(vu));
            p_expr -= vars.times_used.at(vu);
            c_expr -= vars.times_used_C.at(vu);
          }// if
        }// for
        if(layer == 0){
          // on layer 0, flow comes up from s
          DEBUG5(std::cout << " - y_s,"<< I[*u].name);
          p_expr -= vars.from_source.at(*u);
          c_expr -= vars.from_source_C.at(*u);
        } else {
          // on layer 1, flow goes up to tp
          DEBUG5(std::cout << " + y_"<< I[*u].name<<",tp");
          p_expr += vars.to_path_sink.at(*u);
        }// if
        c.add(p_expr == 0);
        c.add(c_expr == 0);
        DEBUG5(std::cout << " == 0"<< std::endl);
      }// for
    }// for
    // flow conservation for layer 2:
    for(auto u = I.get_vertices(); u; ++u){
      IloExpr c_expr(env);
      for(auto e_uv = I.get_incident(*u); e_uv; ++e_uv){
        const ScafVertex& v = I.target(*e_uv);
        const ScafVertexPair vu(v, *u);
        if(!I[*e_uv].is_matching_edge()){
          DEBUG5(std::cout << " - y[2->3]_"<<I.get_edge_name(vu));
          c_expr -= vars.times_used_C_2to3.at(vu);
        }
      }// for
      // on layer 2, flow goes up to tc
      DEBUG5(std::cout << " + y_"<< I[*u].name<<",tc");
      c_expr += vars.to_cycle_sink.at(*u);
      DEBUG5(std::cout << " == 0" << std::endl);
      c.add(c_expr == 0);
    }// for

    // constraint (7): everything that goes to tc has to come in from s
    DEBUG5(std::cout << "adding (7): cycle conservation"<<std::endl);
    for(auto u = I.get_vertices(); u; ++u){
      IloExpr expr(env);
      expr += vars.from_source_C.at(*u);
      expr -= vars.to_cycle_sink.at(*u);
      c.add(expr == 0);
    }

    // constraint (5): each subset S of vertices should contain at most |S|-1 endpoints of arcs
    //DEBUG5(std::cout << "adding (5): each subset S of vertices should contain at most |S|-1 endpoints of arcs"<<std::endl);
    // done via callback!
  }


  // compute the ILP formulation for a graph g
  void populate_model(IloModel& model, 
                      IloEnv& env,
                      VAR_COLLECTION& vars,
#ifdef STATISTICS
                      JumpStatistics jump_stats,
#endif
                      const ILPOptions& opts){
    const Instance& I = vars.I;
    IloRangeArray c(env);
    // construct variables, keeping note of the x_ij, y^l_ij and z^l_ij variables
    DEBUG3(std::cout << "creating variables"<<std::endl);
#ifdef STATISTICS
    populate_variables(model, env, vars, opts, jump_stats);
#else
    populate_variables(model, env, vars, opts);
#endif
    // add optimization function
    DEBUG3(std::cout << "creating optimization expression"<<std::endl);
    IloExpr opt_exp(env);
    for(auto e = I.get_edges(); e; ++e){
      const ScafVertex& u = I.source(*e);
      const ScafVertex& v = I.target(*e);
      // get the appropriate x_ij variable
      auto uv = vars.times_used_undir.find({u,v});
      if(uv == vars.times_used_undir.end()) uv = vars.times_used_undir.find({v,u});
      assert(uv != vars.times_used_undir.end());
      // add the term weight(e)*x_e
      opt_exp += (double)I[*e].weight * uv->second;
    }
    model.add(IloMaximize(env, opt_exp));
    // construct constraints
    DEBUG3(std::cout << "creating constraints"<<std::endl);
    populate_constraints(env, c, vars);
    model.add(c);
  }

}}


