
/** \file convert.hpp
 * converts between scaffold graphs and TOL graphs
 */


#ifndef CONVERT_HPP
#define CONVERT_HPP

#ifdef has_TOL

#include <boost/bimap.hpp>
#include <boost/bimap/unordered_set_of.hpp>

#include "TOL/Graph.h"

#include "utils/utils.hpp"
#include "utils/graph_typedefs.hpp"
#include "utils/scaffolding_typedefs.hpp"

namespace scaffold {

  // a hasher for TOL Vertices
  struct TOL_Vertex_Hash
  {
    size_t operator()(const TOL::Graph::Vertex& v) const {
      return v.index();
    }
  };

  //! bidirectional vertex translation map
  using VTranslate = boost::bimap<boost::bimaps::unordered_set_of<ScafVertex>, boost::bimaps::unordered_set_of<TOL::Graph::Vertex, TOL_Vertex_Hash> >;

  //! convert a graph into the TOL graph format
  void convert(const ScaffoldGraph& g, TOL::Graph& tg, VTranslate* vtranslate = NULL)
  {
    using translate_pair = typename VTranslate::value_type;

    VTranslate* vtrans = vtranslate ? vtranslate : new VTranslate();
    // make sure we can access vtranslate and it's empty
    vtrans->clear();

    // Step 1: copy vertices
    size_t index = 0;
    for(auto r = g.get_vertices(); r; ++r)
      vtrans->insert(translate_pair(*r, (TOL::Graph::Vertex)tg.new_vertex(index++)));

    // Step 2: copy edges
    for(auto e = g.get_edges(); e; ++e)
      tg.new_edge(vtrans->left.at(g.source(*e)), vtrans->left.at(g.target(*e)));

    // clean up what we allocated
    if(!vtranslate) delete vtrans;
  }

}
#endif // has_TOL

#endif // CONVERT_HPP

