

#pragma once

#include "utils/ilp_common.hpp"
#include "utils/ilp_contig_jump_network.hpp"

namespace scaffold {

  // introduce constraints to respect the dependencies of new edges added by the jump preprocessing
  template<typename var_collection_type>
  void respect_jump_dependencies(IloModel& model, const IloEnv& env, const EdgeVarMap& times_used, const ScafVPairDependencyMap& dependencies)
  {
    // the dependency map maps each VertexPair p to a list of VertexPairs on which they depend, that is,
    // p cannot be taken more times than the sum of its dependencies
    IloRangeArray c(env);
    DEBUG5(std::cout << "adding contig-jump-shortcut constraints: if we take an added shortcut, we should also take its dependency"<<std::endl);
    for(const auto& dlist: dependencies){
      const ScafVertexPair& uv = dlist.first;
      const auto& uv_dep_list = dlist.second;
      // build the constraint
      IloExpr expr(env);
      IloExpr rev_expr(env);
      for(const auto& uv_dep_edge: uv_dep_list){
        expr += times_used.at(uv_dep_edge);
        rev_expr += times_used.at(reverse(uv_dep_edge));
      }

      expr -= times_used.at(uv);
      rev_expr -= times_used.at(reverse(uv));
      c.add(expr >= 0);
      c.add(rev_expr >= 0);
    }
    model.add(c);
  }

  /*
  // new contig jump handling: each edge u---v can jump contigs u--x===y--v provided the lengths fit
  void construct_jumps_for_VPair(IloModel& model,
                                const IloEnv& env,
                                var_collection_multi& vars,
                                const ScafVertexPair& uv,
                                const int uv_length,
                                const ScafVertexPairSet& target_set)
  {
    const Instance& I = vars.I;
    assert(uv_length != NO_LENGTH);
    assert(!target_set.empty());
    assert(!I.is_matching_edge(uv));
    
    IloRangeArray c(env);
    const ScafVertex& u = uv.first;
    const ScafVertex& v = uv.second;
    const unsigned uv_multi = I[uv].multiplicity;
    DEBUG5(std::cout << "constructing jump for "<<I[u].name<<"->"<<I[v].name<<std::endl;)

    // for each small contig xy that we can place in the jump, create a variable that is the minimum of ux & yv
    // then, we can take the jump as many times as the sum of all these variables
    IloExpr overall_uv_jumps(env);
    for(const ScafVertexPair& xy: target_set){
      const ScafVertex& x = xy.first;
      const ScafVertex& y = xy.second;
      const unsigned xy_multi = I[xy].multiplicity;

      // step 1: register a variable for the number of times that uv can jump over xy
      IloNumVar uv_jumps_xy(env, 0, xy_multi, IloNumVar::Int);
      // step 2: constrain the number of jumps by how often ux & yv are taken
      IloExpr used_ux(env);
      used_ux += vars.times_used.at(ScafVertexPair(u, x));
      used_ux -= uv_jumps_xy;
      c.add(used_ux >= 0);

      IloExpr used_yv(env);
      used_yv += vars.times_used.at(ScafVertexPair(y, v));
      used_yv -= uv_jumps_xy;
      c.add(used_yv >= 0);

      // step 3: add the number of times uv can jump over xy to the overall number of times that uv can jump
      overall_uv_jumps += uv_jumps_xy;
    }

    const std::string uv_jump_name((std::string)"jump_" + I[u].name + "," + I[v].name);
    const auto uv_jump_var_iter = vars.contig_jumps.emplace(std::piecewise_construct, std::make_tuple(u,v), std::make_tuple(env, 0, uv_multi, IloNumVar::Int, uv_jump_name.c_str())).first;
    overall_uv_jumps -= uv_jump_var_iter->second;
    c.add(overall_uv_jumps >= 0);
 
    model.add(c);
  }// function
*/
}// namespace

