
#pragma once
#include "utils/command_line.hpp"

struct LinearizationOptions {
  //! bool incidating whether we should count deleted edges or cleaned vertices
  bool count_vertices;

  //! bool indicating whether we use the approximation algorithm or not
  bool use_approx;

  //! bool indicating whether we use the brut algorithm or not
  bool use_brute;

  //! bool indicating whether we deconstruct the graph into sequences after linearizing it
  bool deconstruct;

  //! indicating whether we apply the linearization reduction rules
  bool reduction_rules;

  //! filename of a file to export the linearization statistics
  std::string stats_file;

  //! filename of a file to export the solution graph statistics
  std::string solution_graph_stats_file;

  //! bool indicating whether we use the tree decomposition method or not
  bool tw;


};

//! add options to the global options list
void add_linearization_options(po::options_description& desc)
{
  desc.add_options()
    ("cv", "when linearizing, minimize cleaned vertices instead of deleted edges")
    ("app","when linearizing, use approximation algorithm (default: ILP)")
    ("brut", "when linearizing, use the brutal algorithm (default: ILP)")
    ("dec", "deconstruct the graph into sequences after the linearization process")
    ("rrules", "apply the linearization reduction rules on the graph (for readability purposes)")
    ("lstats", po::value<std::string>(), "file to output linearization stats")
    ("sgstats", po::value<std::string>(), "file to output solution graph stats (skip linearization process)")
    ("ltw", "when linearizing, use the tree decomposition method")
            ;

}

//! handle ILP specific options passed by the command-line
LinearizationOptions handle_linearization_options(const po::variables_map& vm)
{
    LinearizationOptions lin_opts;

    lin_opts.count_vertices = vm.count("cv");
    lin_opts.use_approx = vm.count("app");
    lin_opts.use_brute = vm.count("brut");
    lin_opts.deconstruct = vm.count("dec");
    lin_opts.reduction_rules = vm.count("rrules") || vm.count("ltw");
    lin_opts.tw = vm.count("ltw");

    lin_opts.stats_file                       = get_option<std::string>(vm, "lstats", "");
    lin_opts.solution_graph_stats_file        = get_option<std::string>(vm, "sgstats", "");

    return lin_opts;
}

